// Flow rate control for OS syringe pumps
// Made in collaboration with University of Glasgow iGEM 2017

//This code should give you the flow rate you enter into the code for a threaded rod of pitch 0.8mm

// The limits in flow rate [FOR A SYRINGE OF 4.73mm INNER LUMEN DIAMETER] you can use
// (due to the limitations of the microsecond delay function and/or the stepper driver) are:
// Max Flow rate ~32 microlitres per second
// Min Flow rate ~0.4 microlitres per second
// These limits are due to the fact that the stepper motor becomes unstable when the time
// between a step is less than ~ 1000 microseconds and when the delay is greater than ~20000
// microseconds the function itself becomes unstable.

// The parameters which you can control are as follows:
// 1) Flow Rate
// 2) Syringe Inner Lumen Diameter
// 3) Total Distance you want the syringe to be driven

// From iGem Glasgow and Edinburgh UG with love <3

//---------------------------------------------------------------------------
// Pump 0 Variables

// declare pins
int stp_0 = 2;
int dir_0 = 3;
int MS1_0 = 4;
int MS2_0 = 5;
int EN_0 = 6;

// input desired parameters
float flow_rate_0 = 000; //in microliters per second.
float syringe_diam_0 = 000; //diameter of the syringe inner lumen (mm)
float syringe_len_0 = 000;  //total distance user wants syringe to travel (mm)

// variables to be calculated
float interval_0; //delay time for flow rate
int final_num_steps_0;

int step_count_0 = 0;

//---------------------------------------------------------------------------
// Pump 1 Variables

// declare pins
int stp_1 = 7;
int dir_1 = 8;
int MS1_1 = 9;
int MS2_1 = 10;
int EN_1 = 11;

// input desired parameters
float flow_rate_1 = 000; //in microliters per second.
float syringe_diam_1 = 000; //diameter of the syringe inner lumen (mm)
float syringe_len_1 = 000;  //total distance user wants syringe to travel (mm)

// variables to be calculated
float interval_1; //delay time for flow rate
int final_num_steps_1;

int step_count_1 = 0;

//---------------------------------------------------------------------------
// Pump 2 Variables

// declare pins
int stp_2 = 12;
int dir_2 = 13;
int MS1_2 = 14;
int MS2_2 = 15;
int EN_2 = 16;

// input desired parameters
float flow_rate_2 = 000; //in microliters per second.
float syringe_diam_2 = 000; //diameter of the syringe inner lumen (mm)
float syringe_len_2 = 000;  //total distance user wants syringe to travel (mm)

// variables to be calculated
float interval_2; //delay time for flow rate
int final_num_steps_2;

int step_count_2 = 0;

//---------------------------------------------------------------------------

float delayLimit = 20000; //The upper limit of delay time
unsigned long previousMicros = 0;
char user_input;

void setup() {
  // put your setup code here, to run once:

  Serial.begin(9600); //Open Serial connection for debugging
  Serial.println("Begin motor control");
  Serial.println();

  pinMode(stp_0, OUTPUT); pinMode(dir_0, OUTPUT); pinMode(MS1_0, OUTPUT);
  pinMode(MS2_0, OUTPUT); pinMode(EN_0, OUTPUT);

  pinMode(stp_1, OUTPUT); pinMode(dir_1, OUTPUT); pinMode(MS1_1, OUTPUT);
  pinMode(MS2_1, OUTPUT); pinMode(EN_1, OUTPUT);

  pinMode(stp_2, OUTPUT); pinMode(dir_2, OUTPUT); pinMode(MS1_2, OUTPUT);
  pinMode(MS2_2, OUTPUT); pinMode(EN_2, OUTPUT);

  final_num_steps_0 = CalculateNumSteps(syringe_len_0);
  final_num_steps_1 = CalculateNumSteps(syringe_len_1);
  final_num_steps_2 = CalculateNumSteps(syringe_len_2);

  interval_0 = CalculateInterval(flow_rate_0, syringe_diam_0, final_num_steps_0);
  interval_1 = CalculateInterval(flow_rate_1, syringe_diam_1, final_num_steps_1);
  interval_2 = CalculateInterval(flow_rate_2, syringe_diam_2, final_num_steps_2);

  Forward_8th_Step_Mode(0); Forward_8th_Step_Mode(1); Forward_8th_Step_Mode(2);

  Serial.println("Input 1 to begin main Loop");

  while (!Serial.available()) {} //Do nothing until data received over the serial port

}

void loop() {
  // put your main code here, to run repeatedly:

  if (CheckInterval(interval_0) && step_count_0 <= final_num_steps_0) {
    digitalWrite(stp_0, HIGH); //Trigger one step forward
    digitalWrite(stp_0, LOW); //Pull step pin low so it can be triggered again
    step_count_0++;
    Serial.println("Pump 0 step");
  }

  if (CheckInterval(interval_1) && step_count_1 <= final_num_steps_1) {
    digitalWrite(stp_1, HIGH); //Trigger one step forward
    digitalWrite(stp_1, LOW); //Pull step pin low so it can be triggered again
    step_count_1++;
    Serial.println("Pump 1 step");
  }

  if (CheckInterval(interval_2) && step_count_2 <= final_num_steps_2) {
    digitalWrite(stp_2, HIGH); //Trigger one step forward
    digitalWrite(stp_2, LOW); //Pull step pin low so it can be triggered again
    step_count_2++;
    Serial.println("Pump 2 step");
  }

  if (step_count_0 > final_num_steps_0 &&
      step_count_1 > final_num_steps_1 &&
      step_count_2 > final_num_steps_2) {

    Serial.println("Disconnect Pumps from chip then input 1 to reset pumps");

    while (!Serial.available()) {} //Do nothing until data received over the serial port

    Reverse_Step_Mode(0); Reverse_Step_Mode(1); Reverse_Step_Mode(2);

    for (int i = 0; i > step_count_0; i++) {
      digitalWrite(stp_0, HIGH); //Trigger one step forward
      delay(1)
      digitalWrite(stp_0, LOW); //Pull step pin low so it can be triggered again
      delay(1)
    }

    for (int i = 0; i > step_count_1; i++) {
      digitalWrite(stp_1, HIGH); //Trigger one step forward
      delay(1)
      digitalWrite(stp_1, LOW); //Pull step pin low so it can be triggered again
      delay(1)
    }

    for (int i = 0; i > step_count_2; i++) {
      digitalWrite(stp_2, HIGH); //Trigger one step forward
      delay(1)
      digitalWrite(stp_2, LOW); //Pull step pin low so it can be triggered again
      delay(1)
    }
  }

}
